// mozileConfiguration.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *		Lars Kiilerich   <lars[at]v1p[dot]dk>
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Configuration Javascript
Mozile version 0.6.14

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-06-24

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	Injects the eDOM code into the current when told to.

*********/


// Mozile Inject
// Load eDOM and related files into the target document, just like mozileChromeLoader.js

//List of files to inject.
var mozile_js_files = new Array();
mozile_js_files = ["eDOM.js", "eDOMXHTML.js", "domlevel3.js", "mozCE.js", "mozIECE.js"];

function mozileInject(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileInject"];
	mozileDebug(f,2,"Injection starting");
	
	// POST04: if document.documentElement != HTML then ... or no "head" ...
	var head = mozileInjectTarget.document.getElementsByTagName("head")[0];

	if(!head) {
		mozileDebug(f,1,"Injecting into XML");
		head = mozileInjectTarget.document.documentElement;
	}
	
	mozileLoadConfig(mozileInjectTarget, head);
	
	for (var i=0; i < mozile_js_files.length; i++)
	{
		var scr = document.createElementNS(MOZILEC.NAMESPACE_XHTML,"script");
		var src = MOZILEC.EXTENSION_DIR + mozile_js_files[i];
		scr.setAttribute("id", mozile_js_files[i]);
		scr.setAttribute("src", src);
		scr.setAttribute("language","JavaScript");
		head.insertBefore(scr, head.firstChild);
	}

	// Global flag to let you know Mozile has been loaded from chrome.
	mozileInjectTarget.mozileChromeLoaded = "true";
	mozileDebug(f,2,"Injection successful");
}

function mozileScourCSS(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileScourCSS"];
	mozileDebug(f,3,"Scouring CSS");
	mozileDebug(f,3,"Number of style sheets: "+mozileInjectTarget.document.styleSheets.length);
	var URL = "";
	for(var	i=0; i<mozileInjectTarget.document.styleSheets.length;i++)	{
		try{
			mozileDebug(f,3,"Style sheet "+ i +" has "+mozileInjectTarget.document.styleSheets[i].cssRules.length+" rules");
			for	(var j=0; j<mozileInjectTarget.document.styleSheets[i].cssRules.length;j++) {
				var	sel	= mozileInjectTarget.document.styleSheets[i].cssRules[j];
				if(sel.selectorText=="mozileconfiguration")	{
					URL = sel.style.backgroundImage;
					URL = URL.substring(4,URL.length-1);
					mozileDebug(f,3,"URL found:"+i+":"+j+":"+URL);
					break;
				}
			}
		} catch(e) {
			// styleSheet is inaccessible, don't worry about it
			mozileDebug(f,2,"Problem with style sheet number "+i+"? "+e);
		}
	}
	return URL;
}

// returns an absolute URL computed from a (possibly relative) URL and a starting point (relativeTo)
function mozileAbsoluteURL(URL, relativeTo) {
	var f = ["mozileConfiguration.js","mozileAbsoluteURL"];
	mozileDebug(f,2,"Start with: "+relativeTo+"  "+URL);
	if(URL.search("://")>0) {
		mozileDebug(f,2,"URL contains '://', so assume it's absolute.");
		return URL;
	}
	relativeTo += " "; // make sure it's a string

  // Remove query string part of already absolute part
  var queryPos = relativeTo.lastIndexOf("?");
  if (queryPos > -1)
    relativeTo = relativeTo.substring(0, queryPos);

  // Check if given URL is relative to host root or current document
  if (URL.charAt(0) == "/") {
    // Make URL relative
    URL = URL.substring(1);

    // Find beginning of hostname
    var hostPos = relativeTo.search("://")+3;
    if (hostPos < 0)
      hostPos = 0;

    // Find end of hostname
    hostPos = relativeTo.indexOf("/", hostPos);
    relativeTo = relativeTo.substring(0, hostPos) + "/";
  }

	var last = relativeTo.lastIndexOf("/");
  var dir = relativeTo.substring(0,last+1);
  	// just tacking the relative URL onto the directory of the reference URL seems to work fine: i.e. http://overton.ca/james/mozile/mozile1/../mozileTest.html becomes http://overton.ca/james/mozile/mozile1/mozileTest.html
	mozileDebug(f,2,"Try: "+ dir + URL);
	URL = dir+URL;
	return URL;
}

function mozileLoadConfig (mozileInjectTarget, headnode) {
	var f = ["mozileConfiguration.js","mozileLoadConfig"];
	mozileDebug(f,2,"Load Config starting");
	
	// Important Objects, hold all the configuration info for the page
	mozileInjectTarget.document.MOZILE_RSD_CONFIG =	new	Object();
	mozileInjectTarget.document.MOZILE_CURRENT_CONFIG =	new	Object();
	
	/* Look	for	an RSD declaration and load	configuration if present */

	var	rsdtype	= "application/rsd+xml";
	var	rsdns =	"http://archipelago.phrasewise.com/rsd";
	var	methodXPath	= "//setting[@name='saveMethod']";
	var	contentXPath = "//setting[@name='saveContent']";
	var httpSavePathXPath = "//setting[@name='httpSavePath']";
	var	pasteModeXPath = "//setting[@name='pasteMode']";
	var	mesXPath = "//setting[@name='mes']";
	var	tagCSSXPath	= "//setting[@name='tagCSS']";
	var	c, rsdhref, rsdloaded, xmlDoc;
	
	// first check in CSS
	var CSS_URL = mozileScourCSS(mozileInjectTarget);
	if(CSS_URL && CSS_URL!="") {	
		mozileDebug(f,2,"Using configuration from CSS file, href="+CSS_URL);
		rsdhref = CSS_URL;
	} 
	// if that fails, check for a <link>
	else {
		mozileDebug(f,2,"CSS config not found. Checking for an RSD <link...>");	
		// Grab	the	head element and look for the RSD link
		// XXX Moz supports	XPath in HTML now, we could	replace	this w/	XPath
		var	links =	headnode.getElementsByTagName("link");
		for	(var i=0; i	< links.length;	i++) {
			// Find	out	if the link	element	has	a type attr	for	RSD
			var	l =	links[i];
			if (l.hasAttribute("type") && l.getAttribute("type") ==	rsdtype) {
				rsdhref	= l.getAttribute("href");
				mozileDebug(f,2,"RSD link found, href="+rsdhref);
			}
		}
	}
	
	rsdloaded = false;
	if ( rsdhref!=undefined && rsdhref!='' ) {
		rsdhref=mozileAbsoluteURL(rsdhref, mozileInjectTarget.document.location); // make sure the URL is absolute
		mozileDebug(f,2,"Loading RSD at "+rsdhref);
		try {
  		xmlDoc = mozileInjectTarget.document.implementation.createDocument("", "", null);
			xmlDoc.async = false;
			rsdloaded = xmlDoc.load(rsdhref);
			if ( !rsdloaded ) {
					mozileDebug(f,1,"RSD file load has failed");
			}
		} catch (e){
			mozileDebug(f,1,"Exception: "+e);
		}
	}
	
	if ( rsdloaded ) {
		mozileDebug(f,2,"RSD file loaded");

		// If the rsd.xml has a namespace decl, the XPath breaks.
		var	evaluator =	new	XPathEvaluator();
		
		var	saveContent = evaluator.evaluate(contentXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	saveMethod = evaluator.evaluate(methodXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	httpSavePath = evaluator.evaluate(httpSavePathXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	pasteMode	= evaluator.evaluate(pasteModeXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	mes	= evaluator.evaluate(mesXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		var	tagCSS = evaluator.evaluate(tagCSSXPath, xmlDoc, null, XPathResult.STRING_TYPE, null).stringValue;
		
		httpSavePath = mozileAbsoluteURL(httpSavePath,rsdhref);
		if ( mes!="XHTML" ) { // make sure there are absolute URLs
			mes = mozileAbsoluteURL(mes,rsdhref);
		}
		if ( tagCSS!="XHTML" ) {
			tagCSS = mozileAbsoluteURL(tagCSS,rsdhref);
		}

		// Store the values from the RSD file
		c = mozileInjectTarget.document.MOZILE_RSD_CONFIG;
		c.rsdhref = rsdhref;
		c.saveContent =	saveContent;
		c.saveMethod = saveMethod;
		c.httpSavePath = httpSavePath;
		c.pasteMode = pasteMode;
		c.mes = mes;
		c.tagCSS = tagCSS;
	
		// Store the same RSD values to the MOZILE_CURRENT_CONFIG array
		c = mozileInjectTarget.document.MOZILE_CURRENT_CONFIG;
		c.saved = "true";
		c.preset = "page";
		c.rsdhref = rsdhref;
		c.saveContent =	saveContent;
		c.saveMethod = saveMethod;
		c.httpSavePath = httpSavePath;
		c.pasteMode = pasteMode;
		c.mes = mes;
		c.tagCSS = tagCSS;
		
		mozileDebug(f,1,"RSD Configuration completed.");
		mozileLoadMES(mozileInjectTarget, mes);
	}
	else { // default to the built in XHTML Editing Scheme.
		mozileDebug(f,2,"No configuration information found, defaults will be applied");
		// Store the default values in MOZILE_CURRENT_CONFIG.
		c = mozileInjectTarget.document.MOZILE_CURRENT_CONFIG;
		c.saved = "true";
		c.preset = "user";
		c.rsdhref = "";
		c.saveContent =	"";
		c.saveMethod = "";
		c.httpSavePath = "";
		c.pasteMode = "";
		c.mes = "XHTML";
		c.tagCSS = "XHTML";
		mozileLoadMES(mozileInjectTarget,"XHTML");
	}
}
